#region Using declarations
using System;
using System.ComponentModel;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.IO;
using System.Text;
using System.Xml.Serialization;
using NinjaTrader.Cbi;
using NinjaTrader.Data;
using NinjaTrader.Indicator;
using NinjaTrader.Gui.Chart;
using NinjaTrader.Strategy;
#endregion

// This namespace holds all strategies and is required. Do not change it.
namespace NinjaTrader.Strategy
{
    /// <summary>
    /// Enter the description of your strategy here
    /// </summary>
    [Description("Enter the description of your strategy here")]
    public class PStop : Strategy
    {
        #region Variables
        // Wizard generated variables
        private int units = 100000; // Default setting for Units
        private double takeProfit = 500.000; // Default setting for TakeProfit
        private double slTPRatio = 1; // Default setting for StopProbability
        // User defined variables (add any user defined variables below)
        #endregion
		
		private double stopDistance, tpValue, tp, minDistanceSeen;
		
		private StreamReader reader;

        /// <summary>
        /// This method is used to configure the strategy and is called once before any strategy method is called.
        /// </summary>
        protected override void Initialize()
        {
            CalculateOnBarClose = true;
			ExitOnClose = false;
			
			stopDistance =  TakeProfit/SLTPRatio;			
        }
		
		protected override void OnStartUp()
		{
			reader = new StreamReader(@"C:\random.txt");
		}
		
		protected override void OnTermination()
		{
			reader.Close();
		}

        /// <summary>
        /// Called on each bar update event (incoming tick)
        /// </summary>
        protected override void OnBarUpdate()
        {
			tpValue = TickSize * TakeProfit;
			
			double currentTPDistance, newSLDistance;
			
			if( Position.MarketPosition == MarketPosition.Long )
			{
				currentTPDistance = tp - High[0];	
				
				if( currentTPDistance < minDistanceSeen )
				{
					minDistanceSeen = currentTPDistance;
					newSLDistance = minDistanceSeen*SLTPRatio;
					
					SetStopLoss(CalculationMode.Price,High[0]-newSLDistance);
					Logging(minDistanceSeen,newSLDistance,High[0]-newSLDistance);
				}
			}
			else if(Position.MarketPosition == MarketPosition.Short )
			{
				currentTPDistance = Low[0] - tp;	
				
				if( currentTPDistance < minDistanceSeen )
				{
					minDistanceSeen = currentTPDistance;
					newSLDistance = minDistanceSeen*SLTPRatio;
					
					SetStopLoss(CalculationMode.Price,Low[0]+newSLDistance);
					Logging(minDistanceSeen,newSLDistance,Low[0]+newSLDistance);
				}
			}
			
			if( Position.MarketPosition == MarketPosition.Flat )
			{
				Random randomObj = new Random();
				
				int random = Convert.ToInt32( reader.ReadLine() );
				
				if( random > 5000 )
				{
					EnterLong(Units);
					tp = Close[0] + tpValue;
				}
				else
				{
					EnterShort(Units);
					tp = Close[0] - tpValue;
				}
				
				minDistanceSeen = tpValue;
				SetStopLoss(CalculationMode.Ticks, stopDistance);
				SetProfitTarget(CalculationMode.Ticks,TakeProfit);
			}
        }
		
		private void Logging(double minDistanceSeen, double newSLDistance, double sl)
		{
			StringBuilder sb = new StringBuilder("New best price seen at ");
			sb.Append(Time[0]);
			sb.Append(". TP distance = ");
			sb.Append(minDistanceSeen);
			sb.Append(". New SL distance = ");
			sb.Append(newSLDistance);
			sb.Append(". New SL = ");
			sb.Append(sl);
			
			Print(sb.ToString());
		}

        #region Properties
        [Description("")]
        [GridCategory("Parameters")]
        public int Units
        {
            get { return units; }
            set { units = Math.Max(1, value); }
        }

        [Description("")]
        [GridCategory("Parameters")]
        public double TakeProfit
        {
            get { return takeProfit; }
            set { takeProfit = Math.Max(1, value); }
        }

        [Description("")]
        [GridCategory("Parameters")]
        public double SLTPRatio
        {
            get { return slTPRatio; }
            set { slTPRatio = Math.Max(0.01, value); }
        }
        #endregion
    }
}
